import { Ask, Bid, BidShares, EIP712Domain, EIP712Signature, AvatarData } from './types';
import { BigNumber, BigNumberish } from '@ethersproject/bignumber';
import { ContractTransaction } from '@ethersproject/contracts';
import { Provider } from '@ethersproject/providers';
import { Signer } from '@ethersproject/abstract-signer';
import { AvatarExchange, Avatar } from '@motif-foundation/asset/dist/typechain';
export declare class MotifAvatar {
    chainId: number;
    avatarAddress: string;
    avatarExchangeAddress: string;
    signerOrProvider: Signer | Provider;
    avatar: Avatar;
    avatarExchange: AvatarExchange;
    readOnly: boolean;
    constructor(signerOrProvider: Signer | Provider, chainId: number);
    /*********************
     * Motif View Methods
     *********************
     */
    /**
     * Fetches the content hash for the specified avatar on the Motif Avatar Contract
     * @param avatarId
     */
    fetchContentHash(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the metadata hash for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchMetadataHash(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the metadata hash for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchTokenContract(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the content uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchContentURI(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the metadata uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchMetadataURI(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the default for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchIsDefault(avatarId: BigNumberish): Promise<boolean>;
    /**
     * Fetches the creator for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchCreator(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the current bid shares for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchCurrentBidShares(avatarId: BigNumberish): Promise<BidShares>;
    /**
     * Fetches the current ask for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchCurrentAsk(avatarId: BigNumberish): Promise<Ask>;
    /**
     * Fetches the current bid for the specified bidder for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param bidder
     */
    fetchCurrentBidForBidder(avatarId: BigNumberish, bidder: string): Promise<Bid>;
    /**
     * Fetches the permit nonce on the specified avatar id for the owner address
     * @param address
     * @param avatarId
     */
    fetchPermitNonce(address: string, avatarId: BigNumberish): Promise<BigNumber>;
    /**
     * Fetches the current mintWithSig nonce for the specified address
     * @param address
     * @param avatarId
     */
    fetchMintWithSigNonce(address: string): Promise<BigNumber>;
    /*********************
     * Motif Write Methods
     *********************
     */
    /**
     * Updates the content uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param tokenURI
     */
    updateContentURI(avatarId: BigNumberish, tokenURI: string): Promise<ContractTransaction>;
    /**
     * Updates the metadata uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param metadataURI
     */
    updateMetadataURI(avatarId: BigNumberish, metadataURI: string): Promise<ContractTransaction>;
    /**
     * Updates the metadata uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param isDefault
     */
    updateIsDefault(avatarId: BigNumberish, isDefault: boolean): Promise<ContractTransaction>;
    /**
     * Mints a new piece of avatar on an instance of the Motif Avatar Contract
     * @param mintData
     * @param bidShares
     */
    mint(avatarData: AvatarData, bidShares: BidShares): Promise<ContractTransaction>;
    /**
     * Mints a new piece of avatar on an instance of the Motif Avatar Contract
     * @param creator
     * @param avatarData
     * @param bidShares
     * @param sig
     */
    mintWithSig(creator: string, avatarData: AvatarData, bidShares: BidShares, sig: EIP712Signature): Promise<ContractTransaction>;
    /**
     * Sets an ask on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param ask
     */
    setAsk(avatarId: BigNumberish, ask: Ask): Promise<ContractTransaction>;
    /**
     * Sets a bid on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param bid
     */
    setBid(avatarId: BigNumberish, bid: Bid): Promise<ContractTransaction>;
    /**
     * Removes the ask on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    removeAsk(avatarId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Removes the bid for the msg.sender on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    removeBid(avatarId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Accepts the specified bid on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param bid
     */
    acceptBid(avatarId: BigNumberish, bid: Bid): Promise<ContractTransaction>;
    /**
     * Grants the spender approval for the specified avatar using meta transactions as outlined in EIP-712
     * @param sender
     * @param avatarId
     * @param sig
     */
    permit(spender: string, avatarId: BigNumberish, sig: EIP712Signature): Promise<ContractTransaction>;
    /**
     * Revokes the approval of an approved account for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    revokeApproval(avatarId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Burns the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    burn(avatarId: BigNumberish): Promise<ContractTransaction>;
    /***********************
     * ERC-721 View Methods
     ***********************
     */
    /**
     * Fetches the total balance of avatar owned by the specified owner on an instance of the Motif Avatar Contract
     * @param owner
     */
    fetchBalanceOf(owner: string): Promise<BigNumber>;
    /**
     * Fetches the owner of the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchOwnerOf(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches the avatarId of the specified owner by index on an instance of the Motif Avatar Contract
     * @param owner
     * @param index
     */
    fetchAvatarOfOwnerByIndex(owner: string, index: BigNumberish): Promise<BigNumber>;
    /**
     * Fetches the total amount of non-burned avatar that has been minted on an instance of the Motif Avatar Contract
     */
    fetchTotalAvatar(): Promise<BigNumber>;
    /**
     * Fetches the avatarId by index on an instance of the Motif Avatar Contract
     * @param index
     */
    fetchAvatarByIndex(index: BigNumberish): Promise<BigNumber>;
    /**
     * Fetches the approved account for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchApproved(avatarId: BigNumberish): Promise<string>;
    /**
     * Fetches if the specified operator is approved for all avatar owned by the specified owner on an instance of the Motif Avatar Contract
     * @param owner
     * @param operator
     */
    fetchIsApprovedForAll(owner: string, operator: string): Promise<boolean>;
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    /**
     * Grants approval to the specified address for the specified avatar on an instance of the Motif Avatar Contract
     * @param to
     * @param avatarId
     */
    approve(to: string, avatarId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Grants approval for all avatar owner by msg.sender on an instance of the Motif Avatar Contract
     * @param operator
     * @param approved
     */
    setApprovalForAll(operator: string, approved: boolean): Promise<ContractTransaction>;
    /**
     * Transfers the specified avatar to the specified to address on an instance of the Motif Avatar Contract
     * @param from
     * @param to
     * @param avatarId
     */
    transferFrom(from: string, to: string, avatarId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Executes a SafeTransfer of the specified avatar to the specified address if and only if it adheres to the ERC721-Receiver Interface
     * @param from
     * @param to
     * @param avatarId
     */
    safeTransferFrom(from: string, to: string, avatarId: BigNumberish): Promise<ContractTransaction>;
    /****************
     * Miscellaneous
     * **************
     */
    /**
     * Returns the EIP-712 Domain for an instance of the Motif Avatar Contract
     */
    eip712Domain(): EIP712Domain;
    /**
     * Checks to see if a Bid's amount is evenly splittable given the avatar's current bidShares
     *
     * @param avatarId
     * @param bid
     */
    isValidBid(avatarId: BigNumberish, bid: Bid): Promise<boolean>;
    /**
     * Checks to see if an Ask's amount is evenly splittable given the avatar's current bidShares
     *
     * @param avatarId
     * @param ask
     */
    isValidAsk(avatarId: BigNumberish, ask: Ask): Promise<boolean>;
    /**
     * Checks to see if a piece of avatar has verified uris that hash to their immutable hashes
     *
     * @param avatarId
     * @param timeout
     */
    isVerifiedAvatar(avatarId: BigNumberish, timeout?: number): Promise<boolean>;
    /******************
     * Private Methods
     ******************
     */
    /**
     * Throws an error if called on a readOnly == true instance of Motif Sdk
     * @private
     */
    private ensureNotReadOnly;
}
