import { Ask, Bid, BidShares, EIP712Domain, EIP712Signature, ItemData } from './types';
import { BigNumber, BigNumberish } from '@ethersproject/bignumber';
import { ContractTransaction } from '@ethersproject/contracts';
import { Provider } from '@ethersproject/providers';
import { Signer } from '@ethersproject/abstract-signer';
import { ItemExchange, Item } from '@motif-foundation/asset/dist/typechain';
export declare class MotifItem {
    chainId: number;
    itemAddress: string;
    itemExchangeAddress: string;
    signerOrProvider: Signer | Provider;
    item: Item;
    itemExchange: ItemExchange;
    readOnly: boolean;
    constructor(signerOrProvider: Signer | Provider, chainId: number, itemAddress?: string, itemExchangeAddress?: string);
    /*********************
     * Motif View Methods
     *********************
     */
    /**
     * Fetches the content hash for the specified item on the Motif Item Contract
     * @param itemId
     */
    fetchContentHash(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the metadata hash for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchMetadataHash(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the metadata hash for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchTokenContract(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the content uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchContentURI(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the metadata uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchMetadataURI(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the creator for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchCreator(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the current bid shares for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchCurrentBidShares(itemId: BigNumberish): Promise<BidShares>;
    /**
     * Fetches the current ask for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchCurrentAsk(itemId: BigNumberish): Promise<Ask>;
    /**
     * Fetches the current bid for the specified bidder for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bidder
     */
    fetchCurrentBidForBidder(itemId: BigNumberish, bidder: string): Promise<Bid>;
    /**
     * Fetches the permit nonce on the specified item id for the owner address
     * @param address
     * @param itemId
     */
    fetchPermitNonce(address: string, itemId: BigNumberish): Promise<BigNumber>;
    /**
     * Fetches the current mintWithSig nonce for the specified address
     * @param address
     * @param itemId
     */
    fetchMintWithSigNonce(address: string): Promise<BigNumber>;
    /*********************
     * Motif Write Methods
     *********************
     */
    /**
     * Updates the content uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param tokenURI
     */
    updateContentURI(itemId: BigNumberish, tokenURI: string): Promise<ContractTransaction>;
    /**
     * Updates the metadata uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param metadataURI
     */
    updateMetadataURI(itemId: BigNumberish, metadataURI: string): Promise<ContractTransaction>;
    /**
     * Mints a new piece of item on an instance of the Motif Item Contract
     * @param mintData
     * @param bidShares
     */
    mint(itemData: ItemData, bidShares: BidShares): Promise<ContractTransaction>;
    mintMultiple(itemData: Array<ItemData>, bidShares: Array<BidShares>): Promise<ContractTransaction>;
    /**
     * Mints a new piece of item on an instance of the Motif Item Contract
     * @param creator
     * @param itemData
     * @param bidShares
     * @param sig
     */
    mintWithSig(creator: string, itemData: ItemData, bidShares: BidShares, sig: EIP712Signature): Promise<ContractTransaction>;
    /**
     * Sets an ask on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param ask
     */
    setAsk(itemId: BigNumberish, ask: Ask): Promise<ContractTransaction>;
    /**
     * Sets a bid on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bid
     */
    setBid(itemId: BigNumberish, bid: Bid): Promise<ContractTransaction>;
    /**
     * Removes the ask on the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    removeAsk(itemId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Removes the bid for the msg.sender on the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    removeBid(itemId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Accepts the specified bid on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bid
     */
    acceptBid(itemId: BigNumberish, bid: Bid): Promise<ContractTransaction>;
    /**
     * Grants the spender approval for the specified item using meta transactions as outlined in EIP-712
     * @param sender
     * @param itemId
     * @param sig
     */
    permit(spender: string, itemId: BigNumberish, sig: EIP712Signature): Promise<ContractTransaction>;
    /**
     * Revokes the approval of an approved account for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    revokeApproval(itemId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Burns the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    burn(itemId: BigNumberish): Promise<ContractTransaction>;
    /***********************
     * ERC-721 View Methods
     ***********************
     */
    /**
     * Fetches the total balance of item owned by the specified owner on an instance of the Motif Item Contract
     * @param owner
     */
    fetchBalanceOf(owner: string): Promise<BigNumber>;
    /**
     * Fetches the owner of the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchOwnerOf(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches the itemId of the specified owner by index on an instance of the Motif Item Contract
     * @param owner
     * @param index
     */
    fetchItemOfOwnerByIndex(owner: string, index: BigNumberish): Promise<BigNumber>;
    /**
     * Fetches the total amount of non-burned item that has been minted on an instance of the Motif Item Contract
     */
    fetchTotalItem(): Promise<BigNumber>;
    /**
     * Fetches the itemId by index on an instance of the Motif Item Contract
     * @param index
     */
    fetchItemByIndex(index: BigNumberish): Promise<BigNumber>;
    /**
     * Fetches the approved account for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchApproved(itemId: BigNumberish): Promise<string>;
    /**
     * Fetches if the specified operator is approved for all item owned by the specified owner on an instance of the Motif Item Contract
     * @param owner
     * @param operator
     */
    fetchIsApprovedForAll(owner: string, operator: string): Promise<boolean>;
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    /**
     * Grants approval to the specified address for the specified item on an instance of the Motif Item Contract
     * @param to
     * @param itemId
     */
    approve(to: string, itemId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Grants approval for all item owner by msg.sender on an instance of the Motif Item Contract
     * @param operator
     * @param approved
     */
    setApprovalForAll(operator: string, approved: boolean): Promise<ContractTransaction>;
    /**
     * Transfers the specified item to the specified to address on an instance of the Motif Item Contract
     * @param from
     * @param to
     * @param itemId
     */
    transferFrom(from: string, to: string, itemId: BigNumberish): Promise<ContractTransaction>;
    /**
     * Executes a SafeTransfer of the specified item to the specified address if and only if it adheres to the ERC721-Receiver Interface
     * @param from
     * @param to
     * @param itemId
     */
    safeTransferFrom(from: string, to: string, itemId: BigNumberish): Promise<ContractTransaction>;
    /****************
     * Miscellaneous
     * **************
     */
    /**
     * Returns the EIP-712 Domain for an instance of the Motif Item Contract
     */
    eip712Domain(): EIP712Domain;
    /**
     * Checks to see if a Bid's amount is evenly splittable given the item's current bidShares
     *
     * @param itemId
     * @param bid
     */
    isValidBid(itemId: BigNumberish, bid: Bid): Promise<boolean>;
    /**
     * Checks to see if an Ask's amount is evenly splittable given the item's current bidShares
     *
     * @param itemId
     * @param ask
     */
    isValidAsk(itemId: BigNumberish, ask: Ask): Promise<boolean>;
    /**
     * Checks to see if a piece of item has verified uris that hash to their immutable hashes
     *
     * @param itemId
     * @param timeout
     */
    isVerifiedItem(itemId: BigNumberish, timeout?: number): Promise<boolean>;
    /******************
     * Private Methods
     ******************
     */
    /**
     * Throws an error if called on a readOnly == true instance of Motif Sdk
     * @private
     */
    private ensureNotReadOnly;
}
