"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotifItem = void 0;
const tslib_1 = require("tslib");
const Decimal_1 = require("./Decimal");
const abstract_signer_1 = require("@ethersproject/abstract-signer");
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const addresses_1 = require("./addresses");
const utils_1 = require("./utils");
const tiny_invariant_1 = (0, tslib_1.__importDefault)(require("tiny-invariant"));
class MotifItem {
    constructor(signerOrProvider, chainId, itemAddress, itemExchangeAddress) {
        if (!itemAddress != !itemExchangeAddress) {
            (0, tiny_invariant_1.default)(false, 'Motif Constructor: itemAddress and itemExchangeAddress must both be non-null or both be null');
        }
        if (abstract_signer_1.Signer.isSigner(signerOrProvider)) {
            this.readOnly = false;
        }
        else {
            this.readOnly = true;
        }
        this.signerOrProvider = signerOrProvider;
        this.chainId = chainId;
        if (itemAddress && itemExchangeAddress) {
            const parsedItemAddress = (0, utils_1.validateAndParseAddress)(itemAddress);
            const parsedItemExchangeAddress = (0, utils_1.validateAndParseAddress)(itemExchangeAddress);
            this.itemAddress = parsedItemAddress;
            this.itemExchangeAddress = parsedItemExchangeAddress;
        }
        else {
            const network = (0, utils_1.chainIdToNetworkName)(chainId);
            this.itemAddress = addresses_1.addresses[network].item;
            this.itemExchangeAddress = addresses_1.addresses[network].itemExchange;
        }
        this.item = typechain_1.ItemFactory.connect(this.itemAddress, signerOrProvider);
        this.itemExchange = typechain_1.ItemExchangeFactory.connect(this.itemExchangeAddress, signerOrProvider);
    }
    /*********************
     * Motif View Methods
     *********************
     */
    /**
     * Fetches the content hash for the specified item on the Motif Item Contract
     * @param itemId
     */
    fetchContentHash(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenContentHashes(itemId);
        });
    }
    /**
     * Fetches the metadata hash for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchMetadataHash(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenMetadataHashes(itemId);
        });
    }
    /**
     * Fetches the metadata hash for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchTokenContract(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenContractAddresses(itemId);
        });
    }
    /**
     * Fetches the content uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchContentURI(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenURI(itemId);
        });
    }
    /**
     * Fetches the metadata uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchMetadataURI(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenMetadataURI(itemId);
        });
    }
    /**
     * Fetches the creator for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchCreator(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenCreators(itemId);
        });
    }
    /**
     * Fetches the current bid shares for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchCurrentBidShares(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.itemExchange.bidSharesForToken(itemId);
        });
    }
    /**
     * Fetches the current ask for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchCurrentAsk(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.itemExchange.currentAskForToken(itemId);
        });
    }
    /**
     * Fetches the current bid for the specified bidder for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bidder
     */
    fetchCurrentBidForBidder(itemId, bidder) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.itemExchange.bidForTokenBidder(itemId, bidder);
        });
    }
    /**
     * Fetches the permit nonce on the specified item id for the owner address
     * @param address
     * @param itemId
     */
    fetchPermitNonce(address, itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.permitNonces(address, itemId);
        });
    }
    /**
     * Fetches the current mintWithSig nonce for the specified address
     * @param address
     * @param itemId
     */
    fetchMintWithSigNonce(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.mintWithSigNonces(address);
        });
    }
    /*********************
     * Motif Write Methods
     *********************
     */
    /**
     * Updates the content uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param tokenURI
     */
    updateContentURI(itemId, tokenURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(tokenURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.updateTokenURI(itemId, tokenURI);
        });
    }
    /**
     * Updates the metadata uri for the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param metadataURI
     */
    updateMetadataURI(itemId, metadataURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(metadataURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.updateTokenMetadataURI(itemId, metadataURI);
        });
    }
    /**
     * Mints a new piece of item on an instance of the Motif Item Contract
     * @param mintData
     * @param bidShares
     */
    mint(itemData, bidShares) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(itemData.metadataURI);
                (0, utils_1.validateURI)(itemData.tokenURI);
                (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            const gasEstimate = yield this.item.estimateGas.mint(itemData, bidShares);
            const paddedEstimate = gasEstimate.mul(110).div(100);
            return this.item.mint(itemData, bidShares, { gasLimit: paddedEstimate.toString() });
        });
    }
    mintMultiple(itemData, bidShares) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                itemData.map((item, index) => {
                    (0, utils_1.validateURI)(item.metadataURI);
                    (0, utils_1.validateURI)(item.tokenURI);
                    (0, utils_1.validateBidShares)(bidShares[index].creator, bidShares[index].owner, bidShares[index].prevOwner);
                });
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            const gasEstimate = yield this.item.estimateGas.mintMultiple(itemData, bidShares);
            const paddedEstimate = gasEstimate.mul(110).div(100);
            return this.item.mintMultiple(itemData, bidShares, { gasLimit: paddedEstimate.toString() });
        });
    }
    /**
     * Mints a new piece of item on an instance of the Motif Item Contract
     * @param creator
     * @param itemData
     * @param bidShares
     * @param sig
     */
    mintWithSig(creator, itemData, bidShares, sig) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(itemData.metadataURI);
                (0, utils_1.validateURI)(itemData.tokenURI);
                (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.mintWithSig(creator, itemData, bidShares, sig);
        });
    }
    /**
     * Sets an ask on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param ask
     */
    setAsk(itemId, ask) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.setAsk(itemId, ask);
        });
    }
    /**
     * Sets a bid on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bid
     */
    setBid(itemId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.setBid(itemId, bid);
        });
    }
    /**
     * Removes the ask on the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    removeAsk(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.removeAsk(itemId);
        });
    }
    /**
     * Removes the bid for the msg.sender on the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    removeBid(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.removeBid(itemId);
        });
    }
    /**
     * Accepts the specified bid on the specified item on an instance of the Motif Item Contract
     * @param itemId
     * @param bid
     */
    acceptBid(itemId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.acceptBid(itemId, bid);
        });
    }
    /**
     * Grants the spender approval for the specified item using meta transactions as outlined in EIP-712
     * @param sender
     * @param itemId
     * @param sig
     */
    permit(spender, itemId, sig) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.permit(spender, itemId, sig);
        });
    }
    /**
     * Revokes the approval of an approved account for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    revokeApproval(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.revokeApproval(itemId);
        });
    }
    /**
     * Burns the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    burn(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.burn(itemId);
        });
    }
    /***********************
     * ERC-721 View Methods
     ***********************
     */
    /**
     * Fetches the total balance of item owned by the specified owner on an instance of the Motif Item Contract
     * @param owner
     */
    fetchBalanceOf(owner) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.balanceOf(owner);
        });
    }
    /**
     * Fetches the owner of the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchOwnerOf(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.ownerOf(itemId);
        });
    }
    /**
     * Fetches the itemId of the specified owner by index on an instance of the Motif Item Contract
     * @param owner
     * @param index
     */
    fetchItemOfOwnerByIndex(owner, index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenOfOwnerByIndex(owner, index);
        });
    }
    /**
     * Fetches the total amount of non-burned item that has been minted on an instance of the Motif Item Contract
     */
    fetchTotalItem() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.totalSupply();
        });
    }
    /**
     * Fetches the itemId by index on an instance of the Motif Item Contract
     * @param index
     */
    fetchItemByIndex(index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.tokenByIndex(index);
        });
    }
    /**
     * Fetches the approved account for the specified item on an instance of the Motif Item Contract
     * @param itemId
     */
    fetchApproved(itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.getApproved(itemId);
        });
    }
    /**
     * Fetches if the specified operator is approved for all item owned by the specified owner on an instance of the Motif Item Contract
     * @param owner
     * @param operator
     */
    fetchIsApprovedForAll(owner, operator) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.item.isApprovedForAll(owner, operator);
        });
    }
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    /**
     * Grants approval to the specified address for the specified item on an instance of the Motif Item Contract
     * @param to
     * @param itemId
     */
    approve(to, itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.approve(to, itemId);
        });
    }
    /**
     * Grants approval for all item owner by msg.sender on an instance of the Motif Item Contract
     * @param operator
     * @param approved
     */
    setApprovalForAll(operator, approved) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.setApprovalForAll(operator, approved);
        });
    }
    /**
     * Transfers the specified item to the specified to address on an instance of the Motif Item Contract
     * @param from
     * @param to
     * @param itemId
     */
    transferFrom(from, to, itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.transferFrom(from, to, itemId);
        });
    }
    /**
     * Executes a SafeTransfer of the specified item to the specified address if and only if it adheres to the ERC721-Receiver Interface
     * @param from
     * @param to
     * @param itemId
     */
    safeTransferFrom(from, to, itemId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.item.safeTransferFrom(from, to, itemId);
        });
    }
    /****************
     * Miscellaneous
     * **************
     */
    /**
     * Returns the EIP-712 Domain for an instance of the Motif Item Contract
     */
    eip712Domain() {
        // Due to a bug in ganache-core, set the chainId to 1 if its a local blockchain
        // https://github.com/trufflesuite/ganache-core/issues/515
        const chainId = this.chainId == 50 ? 1 : this.chainId;
        return {
            name: 'Motif',
            version: '1',
            chainId: chainId,
            verifyingContract: this.itemAddress,
        };
    }
    /**
     * Checks to see if a Bid's amount is evenly splittable given the item's current bidShares
     *
     * @param itemId
     * @param bid
     */
    isValidBid(itemId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const isAmountValid = yield this.itemExchange.isValidBid(itemId, bid.amount);
            const decimal100 = Decimal_1.Decimal.new(100);
            const currentBidShares = yield this.fetchCurrentBidShares(itemId);
            const isSellOnShareValid = bid.sellOnShare.value.lte(decimal100.value.sub(currentBidShares.creator.value));
            return isAmountValid && isSellOnShareValid;
        });
    }
    /**
     * Checks to see if an Ask's amount is evenly splittable given the item's current bidShares
     *
     * @param itemId
     * @param ask
     */
    isValidAsk(itemId, ask) {
        return this.itemExchange.isValidBid(itemId, ask.amount);
    }
    /**
     * Checks to see if a piece of item has verified uris that hash to their immutable hashes
     *
     * @param itemId
     * @param timeout
     */
    isVerifiedItem(itemId, timeout = 10) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                const [tokenURI, metadataURI, contentHash, metadataHash] = yield Promise.all([
                    this.fetchContentURI(itemId),
                    this.fetchMetadataURI(itemId),
                    this.fetchContentHash(itemId),
                    this.fetchMetadataHash(itemId),
                ]);
                const itemData = (0, utils_1.constructItemData)(tokenURI, metadataURI, contentHash, metadataHash);
                return (0, utils_1.isItemDataVerified)(itemData, timeout);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
        });
    }
    /******************
     * Private Methods
     ******************
     */
    /**
     * Throws an error if called on a readOnly == true instance of Motif Sdk
     * @private
     */
    ensureNotReadOnly() {
        if (this.readOnly) {
            throw new Error('ensureNotReadOnly: readOnly Motif instance cannot call contract methods that require a signer.');
        }
    }
}
exports.MotifItem = MotifItem;
//# sourceMappingURL=MotifItem.js.map