"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotifSpace = void 0;
const tslib_1 = require("tslib");
const Decimal_1 = require("./Decimal");
const abstract_signer_1 = require("@ethersproject/abstract-signer");
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const addresses_1 = require("./addresses");
const utils_1 = require("./utils");
class MotifSpace {
    constructor(signerOrProvider, chainId) {
        if (abstract_signer_1.Signer.isSigner(signerOrProvider)) {
            this.readOnly = false;
        }
        else {
            this.readOnly = true;
        }
        this.signerOrProvider = signerOrProvider;
        this.chainId = chainId;
        const network = (0, utils_1.chainIdToNetworkName)(chainId);
        this.spaceAddress = addresses_1.addresses[network].space;
        this.spaceExchangeAddress = addresses_1.addresses[network].spaceExchange;
        this.space = typechain_1.SpaceFactory.connect(this.spaceAddress, signerOrProvider);
        this.spaceExchange = typechain_1.SpaceExchangeFactory.connect(this.spaceExchangeAddress, signerOrProvider);
    }
    /*********************
     * Space Read Methods
     *********************
     */
    fetchContentHash(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenContentHashes(spaceId);
        });
    }
    fetchMetadataHash(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenMetadataHashes(spaceId);
        });
    }
    fetchContentURI(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenURI(spaceId);
        });
    }
    fetchMetadataURI(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenMetadataURI(spaceId);
        });
    }
    fetchIsPublic(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenIsPublic(spaceId);
        });
    }
    fetchPin(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenPin(spaceId);
        });
    }
    fetchLands(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.lands(spaceId);
        });
    }
    fetchCreator(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenCreators(spaceId);
        });
    }
    fetchCurrentBidShares(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.spaceExchange.bidSharesForToken(spaceId);
        });
    }
    fetchCurrentAsk(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.spaceExchange.currentAskForToken(spaceId);
        });
    }
    fetchCurrentBidForBidder(spaceId, bidder) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.spaceExchange.bidForTokenBidder(spaceId, bidder);
        });
    }
    fetchPermitNonce(address, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.permitNonces(address, spaceId);
        });
    }
    fetchMintWithSigNonce(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.mintWithSigNonces(address);
        });
    }
    // /*********************
    //  *  Space Write Methods
    //  *********************
    //  */
    updateContentURI(spaceId, tokenURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(tokenURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.updateTokenURI(spaceId, tokenURI);
        });
    }
    updateMetadataURI(spaceId, metadataURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(metadataURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.updateTokenMetadataURI(spaceId, metadataURI);
        });
    }
    /* public async updatePublicity(  //TODO LATER
      spaceId: BigNumberish,
      isPublic: boolean
    ): Promise<ContractTransaction> {
      try {
        this.ensureNotReadOnly()
      } catch (err) {
        return Promise.reject(err.message)
      }
  
      return this.space.updateTokenPublicity(spaceId, isPublic)
    }
    
    function updateTokenLands( //TODO LATER*/
    mint(spaceData, bidShares) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(spaceData.metadataURI);
                (0, utils_1.validateURI)(spaceData.tokenURI);
                (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            const gasEstimate = yield this.space.estimateGas.mint(spaceData, bidShares);
            const paddedEstimate = gasEstimate.mul(110).div(100);
            return this.space.mint(spaceData, bidShares, {
                gasLimit: paddedEstimate.toString(),
            });
        });
    }
    setAsk(spaceId, ask) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.setAsk(spaceId, ask);
        });
    }
    setBid(spaceId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.setBid(spaceId, bid);
        });
    }
    removeAsk(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.removeAsk(spaceId);
        });
    }
    removeBid(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.removeBid(spaceId);
        });
    }
    acceptBid(spaceId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.acceptBid(spaceId, bid);
        });
    }
    revokeApproval(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.revokeApproval(spaceId);
        });
    }
    burn(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.burn(spaceId);
        });
    }
    // **********************
    //  * ERC-721 Read Methods
    //  ***********************
    fetchBalanceOf(owner) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.balanceOf(owner);
        });
    }
    fetchOwnerOf(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.ownerOf(spaceId);
        });
    }
    fetchSpaceOfOwnerByIndex(owner, index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenOfOwnerByIndex(owner, index);
        });
    }
    fetchTotalSpace() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.totalSupply();
        });
    }
    fetchSpaceByIndex(index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenByIndex(index);
        });
    }
    fetchApproved(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.getApproved(spaceId);
        });
    }
    fetchIsApprovedForAll(owner, operator) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.isApprovedForAll(owner, operator);
        });
    }
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    approve(to, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.approve(to, spaceId);
        });
    }
    setApprovalForAll(operator, approved) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.setApprovalForAll(operator, approved);
        });
    }
    transferFrom(from, to, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.transferFrom(from, to, spaceId);
        });
    }
    safeTransferFrom(from, to, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.safeTransferFrom(from, to, spaceId);
        });
    }
    /****************
     * Miscellaneous
     * **************
     */
    isValidBid(spaceId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const isAmountValid = yield this.spaceExchange.isValidBid(spaceId, bid.amount);
            const decimal100 = Decimal_1.Decimal.new(100);
            const currentBidShares = yield this.fetchCurrentBidShares(spaceId);
            const isSellOnShareValid = bid.sellOnShare.value.lte(decimal100.value.sub(currentBidShares.creator.value));
            return isAmountValid && isSellOnShareValid;
        });
    }
    isValidAsk(spaceId, ask) {
        return this.spaceExchange.isValidBid(spaceId, ask.amount);
    }
    isVerifiedSpace(spaceId, timeout = 10) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                const [tokenURI, metadataURI, contentHash, metadataHash, isPublic, lands, pin] = yield Promise.all([
                    this.fetchContentURI(spaceId),
                    this.fetchMetadataURI(spaceId),
                    this.fetchContentHash(spaceId),
                    this.fetchMetadataHash(spaceId),
                    this.fetchIsPublic(spaceId),
                    this.fetchLands(spaceId),
                    this.fetchPin(spaceId),
                ]);
                const spaceData = (0, utils_1.constructSpaceData)(tokenURI, metadataURI, contentHash, metadataHash, isPublic, lands, pin);
                return (0, utils_1.isSpaceDataVerified)(spaceData, timeout);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
        });
    }
    /******************
     * Private Methods
     ******************
     */
    ensureNotReadOnly() {
        if (this.readOnly) {
            throw new Error('ensureNotReadOnly: readOnly Motif instance cannot call contract methods that require a signer.');
        }
    }
}
exports.MotifSpace = MotifSpace;
//# sourceMappingURL=MotifSpace.js.map